/*
 * defines.h: All macros for the plugin
 */

/*
 * Spectrum Analyzer plugin for VDR (C++)
 *
 * (C) 2006 Christian Leuschen <christian.leuschen@gmx.de>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#ifndef ___DEFINES_H
#define ___DEFINES_H


#define SPAN_SET_PCM_DATA_ID "Span-SetPcmData-v1.0"
#define SPAN_GET_BAR_HEIGHTS_ID "Span-GetBarHeights-v1.0"

// SpanData
struct Span_SetPcmData_1_0 {
	int length; // the length of the PCM-data
	int *data;	// the PCM-Data as 32-bit int, however only the lower 16-bit are used
				// and you have to take care to hand in such data!
};

struct Span_GetBarHeights_v1_0 {
	int bands;						// number of bands to compute
	int *barHeights;				// the heights of the bars of the two channels combined
	int *barHeightsLeftChannel;		// the heights of the bars of the left channel
	int *barHeightsRightChannel;	// the heights of the bars of the right channel
	int *volumeLeftChannel;			// the volume of the left channels
	int *volumeRightChannel;		// the volume of the right channels
	int *volumeBothChannels;		// the combined volume of the two channels
	const char *name;				// name of the plugin that wants to get the data
									// (must be unique for each client!)
	int falloff;                    // bar falloff value
	// barPeaks[peak-value]
	// it is planned to support peaks falling as if they were subject to gravity
	// NOT supported yet!
	int *barPeaks;
};

// Define a "nice" (human understandable) hight of the bars.
// So each client may scale the heights with the guarantee that it is represented as a percentage of 100%.
#define HEIGHT 100

// TODO: Check the following:
// yes, FFTW_SIZE should be at least as big as PCM_FRAME_SIZE/4 (=2048/4) optimally
// however, for slower CPUs (<200MHz) half of it is enough. Then some data to analyze is omitted.
#define FFTW_SIZE 1024
//#define FFTW_SIZE_MIN 512
#define NUM_BANDS (FFTW_SIZE/2)

// DATA_TIMEOUT = max. mseconds without requests for visualisation data of a client, after that time the
// data shall be cleared
#define DATA_TIMEOUT 2000

#endif //___DEFINES_H
